/*
 * ShareMeImpl.java
 *
 * Created on 13 March 2003, 20:52
 */

package at.ac.tuwien.infosys.rnue.implementation;

/**
 *
 * @author  therapy
 */

import java.util.Properties;
import java.util.Timer;
import java.util.TimerTask;
import at.ac.tuwien.infosys.rnue.helpers.ShareMeException;
import at.ac.tuwien.infosys.rnue.implementation.multicast.IsAliveReceiver;
import at.ac.tuwien.infosys.rnue.implementation.multicast.IsAliveSender;
import at.ac.tuwien.infosys.rnue.implementation.multicast.HostListImpl;
import at.ac.tuwien.infosys.rnue.implementation.multicast.ShutdownListener;
import at.ac.tuwien.infosys.rnue.interfaces.IConstants;
import at.ac.tuwien.infosys.rnue.helpers.search.*;
import at.ac.tuwien.infosys.rnue.helpers.http.*;
import at.ac.tuwien.infosys.rnue.interfaces.ISearchRequest;
import at.ac.tuwien.infosys.rnue.interfaces.*;
import at.ac.tuwien.infosys.rnue.implementation.search.*;
import at.ac.tuwien.infosys.rnue.implementation.security.*;
import at.ac.tuwien.infosys.rnue.implementation.http.*;
import java.security.*;

import java.util.*;
import java.rmi.registry.LocateRegistry;

public class ShareMeImpl implements at.ac.tuwien.infosys.rnue.interfaces.IShareMe {
    
    Properties props;
    IsAliveReceiver myIAR;
    HostListImpl hostlist = new HostListImpl();
    Timer timer;
    SearchEngineImpl sei;
    String myrmiurl; 
    ISecurityHelper sechelp;
    HTTPServerImpl httpserver = null;
    
    /** Creates a new instance of ShareMeImpl */
    public ShareMeImpl(Properties props) throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        try
        {
            props.put(IConstants.RMI_REGISTRY_HOST,java.net.InetAddress.getLocalHost().getHostAddress());
        } catch(java.net.UnknownHostException e)
        {
           throw new ShareMeException("Unable to determinate local IP");
        }
        this.props = props;
        myrmiurl = "rmi://" + props.getProperty(IConstants.RMI_REGISTRY_HOST) + ":" +  
        props.getProperty(IConstants.RMI_REGISTRY_PORT) + "/" + props.getProperty(IConstants.RMI_SERVICE_NAME);
        sechelp = new SecurityHelperImpl(props);
    }
    
    public at.ac.tuwien.infosys.rnue.interfaces.ISearchResult search(String str) throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        
        if(str == null) throw new ShareMeException("Null search string");

        
        ISearchRequest ireq = new SearchRequestImpl(str,sechelp.sign(str),props.getProperty(IConstants.HUMAN_READABLE_NAME));
        SearchResultImpl ires = new SearchResultImpl();
        Enumeration enum = hostlist.elements();
        while(enum.hasMoreElements())
        {
            IHostInfo ih = null;
            try {
                ih = (IHostInfo)enum.nextElement();
                String rmiurl = "rmi://" +  ih.getHostInfoMessage().getRegistryHost() + ":" +
                                        ih.getHostInfoMessage().getRegistryPort() + "/" + 
                                        ih.getHostInfoMessage().getServiceName();
                // System.out.println(rmiurl);                            
                
                ISearchEngine ise = (ISearchEngine)java.rmi.Naming.lookup(rmiurl);
                ISearchResponse isr = null;
                try {
                    isr = ise.search(ireq);
                } catch(Exception e)
                {
                    if(ih != null)
                        System.out.println(ih.getHostInfoMessage().getHumanReadableName() + " throws " + e.getClass().toString() + ": " + e.getMessage());
                    continue;
                }
                
                if(isr == null || isr.getFileList() == null) 
                {       
                    System.out.println("Implementation sanity check failed for " + ih.getHostInfoMessage().getHumanReadableName());
                    continue;
                }
                if(isr.getSignature() != null)
                {
                    try {
                        PublicKey pk = sechelp.getPublicKeyFromCA(ih.getHostInfoMessage().getHumanReadableName());
                        if(sechelp.verify(isr.getFileList(), isr.getSignature(), pk))
                            ires.putFileList(ih.getHostInfoMessage().getHumanReadableName(), isr.getFileList());
                        else
                            System.out.println(ih.getHostInfoMessage().getHumanReadableName() + " has provided a corrupt signature, ignored.");
                    } catch(ShareMeException e)
                        {       
                            System.out.println(ih.getHostInfoMessage().getHumanReadableName() + " has provided a not-verifiable signature, ignored.");
                    }
                }
                else
                    System.out.println(ih.getHostInfoMessage().getHumanReadableName() + " hasn't provided a signature, ignored.");
            } catch(java.rmi.RemoteException e)
            { 
              if(ih != null)
                  System.out.println(ih.getHostInfoMessage().getHumanReadableName() + " has troubles, RemoteException: " + e.getMessage());
            } catch(java.rmi.NotBoundException e)
            { 
              if(ih != null)
                  System.out.println(ih.getHostInfoMessage().getHumanReadableName() + " hasn't bound it's RMI object: " + e.getMessage());
            } catch(java.net.MalformedURLException e)
            { e.printStackTrace();
            }
        }
        return ires;
    }
    
    public void start() throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        System.out.println("This is an importent message.");
        new Thread(
            new ShutdownListener(this,
                                 Integer.parseInt(props.getProperty(IConstants.SHUTDOWN_PORT)),
                                 props.getProperty(IConstants.SHUTDOWN_PASSWORD))
                  ).start();
        myIAR = new IsAliveReceiver(props, hostlist);
        new Thread(myIAR).start();
        IsAliveSender ttask = new IsAliveSender(props);
        timer = new Timer();
        timer.scheduleAtFixedRate(ttask, 5000, IConstants.IS_ALIVE_INTERVAL);
        try {
            java.rmi.registry.LocateRegistry.createRegistry(Integer.parseInt(props.getProperty(IConstants.RMI_REGISTRY_PORT)));
            sei = new SearchEngineImpl(props.getProperty(IConstants.FILE_BASE),
                                                       "http://" + props.getProperty(IConstants.RMI_REGISTRY_HOST) + ":" +
                                                       props.getProperty(IConstants.HTTP_SERVER_PORT) + 
                                                       IConstants.DOWNLOAD_PREFIX,
                                                       sechelp);
            java.rmi.Naming.rebind(myrmiurl, sei);
        } catch(Exception e)
        {
            e.printStackTrace();
        }
        httpserver = new HTTPServerImpl(new Integer(Integer.parseInt(props.getProperty(IConstants.HTTP_SERVER_PORT))));
        httpserver.register(IConstants.DOC_PREFIX, new DocumentEntry(props.getProperty(IConstants.DOCUMENT_BASE)));
        httpserver.register(IConstants.DOWNLOAD_PREFIX, new DocumentEntry(props.getProperty(IConstants.FILE_BASE)));
        httpserver.register(IConstants.CGIBIN_PREFIX, new ShareMeEntry(this));
        new Thread(httpserver).start();
    }
    
    public void stop() throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        if(myIAR != null) 
            myIAR.stop();
        hostlist.stopGarbageCollector();
        if(timer != null)
            timer.cancel();
        try
        {
            java.rmi.Naming.unbind(myrmiurl);
            java.rmi.server.UnicastRemoteObject.unexportObject(sei,true);
        } catch(Exception e)
        { e.printStackTrace(); //FIXME
        }
        if(httpserver != null) 
            httpserver.stop();
    }
}
