/*
 * HTTPServerImpl.java
 *
 * Created on 19 May 2003, 17:56
 */

package at.ac.tuwien.infosys.rnue.implementation.http;

import java.net.*;
import java.util.*;
import java.io.*;
import at.ac.tuwien.infosys.rnue.interfaces.*;
import at.ac.tuwien.infosys.rnue.helpers.http.*;
import at.ac.tuwien.infosys.rnue.helpers.*;

/**
 *
 * @author  therapy
 */
public class HTTPServerImpl implements at.ac.tuwien.infosys.rnue.interfaces.IHTTPServer {
    
    ServerSocket ss;
    boolean running = false;
    Hashtable names = new Hashtable();
    
    /** Creates a new instance of HTTPServerImpl */
    public HTTPServerImpl(Integer i) throws ShareMeException {
        try {
            ss = new ServerSocket(i.intValue());
            System.out.println(ss.toString());
            ss.setSoTimeout(IConstants.HTTP_SERVER_TIMEOUT);
            running = true;
        } catch(Exception e)
        {   e.printStackTrace();
            throw new ShareMeException(e.getMessage());
        }
    }
    
    public void register(String str, at.ac.tuwien.infosys.rnue.interfaces.IHTTPServerEntry iHTTPServerEntry) throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        names.put(str, iHTTPServerEntry);
    }
    
    /** When an object implementing interface <code>Runnable</code> is used
     * to create a thread, starting the thread causes the object's
     * <code>run</code> method to be called in that separately executing
     * thread.
     * <p>
     * The general contract of the method <code>run</code> is that it may
     * take any action whatsoever.
     *
     * @see     java.lang.Thread#run()
     *
     */
    public void run() {
        System.out.println("Running");
        while(running)
        {
            try {
                Socket s = ss.accept();
                new HTTPServerImpl.RequestHandlerThread(s).run();
            } catch(java.net.SocketTimeoutException e)
            { // ignore it
            } catch(Exception e)
            { e.printStackTrace(); //FIXME
            }
        }
        try {
            ss.close();
        } catch(Exception e)
        { e.printStackTrace(); //FIXME
        }
    }
    
    public void stop() {
    }
    
    public void unregister(String str) throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        names.remove(str);
    }
    
    private String errorCode(int code)
    {
        switch(code)
        {
            case 200: return "200 OK";
            case 400: return "400 Bad Request";
            case 404: return "404 Not Found";
            case 501: return "501 Method Not Implemented";
        }
        return "500 Internal Server Error";
    }
    
    public class RequestHandlerThread extends java.lang.Thread {
        Socket s;
        
        public RequestHandlerThread(Socket s)
        {
            this.s = s;
        }
        
        public void run()
        {
            IDocument id = new SimpleDocument("Internal Error",false);
            int code = 500;
            // worst code ever written
            try {
                try {
                    BufferedReader reader = new BufferedReader(new InputStreamReader(s.getInputStream()));

                    String first = reader.readLine();
                    String url,protocol;
                    try
                    {   StringTokenizer strok = new StringTokenizer(first);
                        if(!strok.nextToken().equals("GET")) 
                        {
                            code = 501;
                            throw new HTTPException(501, "Method not implemented");
                        }

                        url = URLDecoder.decode(strok.nextToken(),"UTF-8");
                        protocol = strok.nextToken();
                    } catch(NoSuchElementException e)
                    {
                        throw new HTTPException(400,"Malformed Request");
                    }

                    String thisline; 
                    while(!(thisline = reader.readLine()).equals(""))
                    {
                        if(thisline.startsWith("Host:"))
                            break;
                    }
                    if(!protocol.equals("HTTP/1.0") && !(protocol.equals("HTTP/1.1") && thisline.startsWith("Host:" ))) 
                        throw new HTTPException(400,"Malformed Request"); 

                    Enumeration e = names.keys();
                    int flen = 0;
                    IHTTPServerEntry ise = null;
                    while(e.hasMoreElements())
                    {
                        String s = (String)e.nextElement();
                        if(s.length() > flen && url.startsWith(s))
                        {
                            ise = (IHTTPServerEntry)names.get(s);
                            flen = s.length();
                        }
                    }
                    if(ise == null)
                        throw new HTTPException(404,"File not found");

                    id = ise.getDocument(url.substring(flen,url.length()));
                    code = 200;
                } catch(HTTPException e)
                {
                    code = e.getCode();
                    id = new SimpleDocument(e.getErrorDocument(),false);
                } catch(java.io.IOException e)
                {
                    return;
                } catch(ShareMeException e)
                {
                    e.printStackTrace();
                    return;
                }
                try {
                    OutputStream os = s.getOutputStream();
                    InputStream is = id.getContent();
                    String header = "HTTP/1.1 " + errorCode(code) + "\nDate: " + RFC1123DateFormatter.format(new java.util.Date()) + "\nServer: ShareMe.therp.UGABUGA-RC1\nContent-Length: " +
                    id.getContentLength() + "\nContent-Type: " + id.getContentType() + "\n\n";
                    os.write(header.getBytes("UTF-8"));

                    byte buf[] = new byte[1000];
                    while(is.read(buf) != -1)
                    {
                        os.write(buf);
                    }
                } catch(java.io.IOException e)
                {   
                }
            } finally {
                try {
                    s.close();
                } catch(java.io.IOException e)
                {
                }
            }
        }
    }
}
