/*
 * IsAliveReceiver.java
 *
 * Created on 13 March 2003, 22:17
 */

package at.ac.tuwien.infosys.rnue.implementation.multicast;

/**
 *
 * @author  therapy
 */

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.net.MulticastSocket;
import java.net.InetAddress;
import java.net.DatagramPacket;
import java.util.Properties;
import at.ac.tuwien.infosys.rnue.interfaces.IHostList;
import at.ac.tuwien.infosys.rnue.interfaces.IHostInfoMessage;
import at.ac.tuwien.infosys.rnue.interfaces.IConstants;
import at.ac.tuwien.infosys.rnue.helpers.ShareMeException;
import at.ac.tuwien.infosys.rnue.helpers.multicast.HostInfoImpl;

public class IsAliveReceiver implements at.ac.tuwien.infosys.rnue.interfaces.IIsAliveReceiver {
    
    MulticastSocket msocket;
    InetAddress mcastaddr;
    IHostList ihl;
    boolean running;
    
    /** Creates a new instance of IsAliveReceiver */
    public IsAliveReceiver(Properties props, IHostList ihl) throws ShareMeException {
        try {
            msocket = new MulticastSocket(Integer.parseInt(props.getProperty(IConstants.MULTICAST_PORT)));
            msocket.setSoTimeout(IConstants.IS_ALIVE_RECEIVER_TIMEOUT);
            mcastaddr = InetAddress.getByName(props.getProperty(IConstants.MULTICAST_ADDRESS));
            this.ihl = ihl;
        } catch(java.io.IOException e) {
            throw new ShareMeException("Can't create socket");
        }
    }
    
    /** When an object implementing interface <code>Runnable</code> is used
     * to create a thread, starting the thread causes the object's
     * <code>run</code> method to be called in that separately executing
     * thread.
     * <p>
     * The general contract of the method <code>run</code> is that it may
     * take any action whatsoever.
     *
     * @see     java.lang.Thread#run()
     *
     */
    public void run() {
        byte buf[] = new byte[IConstants.MAX_ISALIVE_PACKET_LENGTH];
        DatagramPacket dp = new DatagramPacket(buf,buf.length);
        running = true;
        try {
                msocket.joinGroup(mcastaddr);
                while(running)
                {
                    try {
                        Object obj;
                        msocket.receive(dp);
                        obj = new ObjectInputStream(new ByteArrayInputStream(dp.getData())).readObject();
                        if(obj instanceof IHostInfoMessage)
                        {  
                            IHostInfoMessage im = (IHostInfoMessage)obj;
                            HostInfoImpl himpl = new HostInfoImpl();
                            himpl.setHostInfoMessage(im);
                            himpl.setTimestamp(System.currentTimeMillis());
                            ihl.put(himpl);
//                            System.out.println(im.getHumanReadableName() + ";" + im.getServiceName());
                        }
                    } catch(java.io.IOException e) {
                     // Intentionally ignore it 
                    } catch(java.lang.ClassNotFoundException e) {
                       e.printStackTrace();
                    }
                }
                msocket.leaveGroup(mcastaddr);
        } catch (java.io.IOException e) {
            e.printStackTrace();
            // terminate thread
        }
    }
    
    public void stop() throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
            running = false;
    }
    
}
