/*
 * CryptographyImpl.java
 *
 * Created on 11 May 2003, 15:55
 */

package at.ac.tuwien.infosys.rnue.implementation.security;

import at.ac.tuwien.infosys.rnue.helpers.*;
import at.ac.tuwien.infosys.rnue.interfaces.*;
import java.security.*;
import java.io.*;

/**
 *
 * @author  therapy
 */
public class CryptographyImpl implements at.ac.tuwien.infosys.rnue.interfaces.ICryptography {
    
    Signature sig;
    KeyPair kp;
    
    /** Creates a new instance of CryptographyImpl */
    public CryptographyImpl() throws ShareMeException {
        try { 
            sig = Signature.getInstance("MD5withRSA");
            
        } catch(java.security.NoSuchAlgorithmException e)
        { throw new ShareMeException("MD5withRSA Algorithm not found.");
        }
    }
    
    public byte[] convertSerializableToByteArray(java.io.Serializable serializable) throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try {
            ObjectOutputStream oos = new ObjectOutputStream(baos);
            oos.writeObject(serializable);
            oos.close();
        } catch(java.io.IOException e)
        { e.printStackTrace();
          throw new ShareMeException("I/O Error.");
        }
        return baos.toByteArray();
    }
    
    public java.security.PublicKey initialize(String str, String keyFile) throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        File keyfile = new File(keyFile);
        if(keyfile.exists())
        {
            try { 
                ObjectInputStream ois = new ObjectInputStream(new FileInputStream(keyfile));
                Object o = ois.readObject();
                if(o instanceof KeyPair)
                {
                   kp = (KeyPair)o;
                }
                ois.close();
            } catch(java.io.IOException e)
            {   throw new ShareMeException("I/O Exception while reading keyfile.");
            } catch(java.lang.ClassNotFoundException e)
            {   e.printStackTrace();
                throw new ShareMeException();
            }
            return null;
        } else
        {
            try { 
                KeyPairGenerator kpg = KeyPairGenerator.getInstance("RSA");
                kpg.initialize(IConstants.KEY_LENGTH);
                kp = kpg.generateKeyPair();
                ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(keyfile));
                oos.writeObject(kp);
                oos.close();
            } catch(java.io.IOException e)
            {   throw new ShareMeException("I/O Exception while writing keyfile.");
            } catch(java.security.NoSuchAlgorithmException e)
            {   throw new ShareMeException("NoSuchAlgorithmException");
            }
            return kp.getPublic();
        }
    }
    
    public byte[] sign(java.io.Serializable serializable) throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        try { 
            synchronized(sig) {
                sig.initSign(kp.getPrivate());
                sig.update(convertSerializableToByteArray(serializable));
                return sig.sign();
            }
        } catch(java.security.SignatureException e)
        {
            e.printStackTrace();
            throw new ShareMeException();
        } catch(java.security.InvalidKeyException e)
        {
            e.printStackTrace();
            throw new ShareMeException();
        }
    }
    
    public boolean verify(java.io.Serializable serializable, byte[] values, java.security.PublicKey publicKey) throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        try { 
            synchronized(sig) {
                sig.initVerify(publicKey);
                sig.update(convertSerializableToByteArray(serializable));
                return sig.verify(values);
            }
        } catch(java.security.SignatureException e)
        {
            e.printStackTrace();
            throw new ShareMeException();
        } catch(java.security.InvalidKeyException e)
        {
            e.printStackTrace();
            throw new ShareMeException();
        }
    }   
}
