/*
 * SecurityHelperImpl.java
 *
 * Created on 11 May 2003, 18:16
 */

package at.ac.tuwien.infosys.rnue.implementation.security;

import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import java.util.*;
import at.ac.tuwien.infosys.rnue.interfaces.*;
import at.ac.tuwien.infosys.rnue.helpers.*;
import at.ac.tuwien.infosys.rnue.implementation.security.CorbaCAImpl.*;
import java.security.*;
import java.io.*;

/**
 *
 * @author  therapy
 */
public class SecurityHelperImpl implements at.ac.tuwien.infosys.rnue.interfaces.ISecurityHelper {
    
    ICertificationAuthority ica;
    CryptographyImpl cryptoimpl;
    
    /** Creates a new instance of SecurityHelperImpl */
    public SecurityHelperImpl(Properties props) {
        try {
            // create and init the ORB 
            ORB orb = ORB.init(new String[]
                      {"-ORBInitialHost", props.getProperty(IConstants.ORB_HOST), 
                       "-ORBInitialPort", props.getProperty(IConstants.ORB_PORT)}, null);

            // get the root naming context
            org.omg.CORBA.Object objRef = 
                orb.resolve_initial_references("NameService");
            NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

            // lookup the ca instance in naming
            this.ica = ICertificationAuthorityHelper.narrow(ncRef.resolve_str(IConstants.DSG_CA_NAME));
            System.out.println("Server object resolved.");

            cryptoimpl = new CryptographyImpl();
            PublicKey pk = 
            cryptoimpl.initialize(props.getProperty(IConstants.HUMAN_READABLE_NAME), 
                                     props.getProperty(IConstants.KEYFILE_NAME));
            if( pk != null)
                registerPublicKey(props.getProperty(IConstants.HUMAN_READABLE_NAME),pk);
            
        } catch(Exception e)
        {
            e.printStackTrace(); // FIXME
        }
    }
    
    public java.security.PublicKey getPublicKeyFromCA(String str) throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        try {
        ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(ica.getKey(str)));
        java.lang.Object o = ois.readObject();
        if(o instanceof PublicKey)
            return (PublicKey)o;
        } catch(java.io.IOException e)
        { 
        } catch(java.lang.ClassNotFoundException e)
        { 
        } catch(at.ac.tuwien.infosys.rnue.implementation.security.CorbaCAImpl.CorbaCAException e)
        { throw new ShareMeException("CA can't supply PublicKey for " + str);
        }
        throw new ShareMeException("Cannot retrieve PublicKey for " + str);
    }
    
    public boolean registerPublicKey(String str, java.security.PublicKey publicKey) throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        try {
        ica.registerKey(str, cryptoimpl.convertSerializableToByteArray(publicKey),"foobar");
        } catch(Exception e)
        {
            e.printStackTrace(); //FIXME
        }
        return false;
    }
    
    public byte[] sign(java.io.Serializable serializable) throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        return cryptoimpl.sign(serializable);
    }
    
    public boolean verify(java.io.Serializable serializable, byte[] values, java.security.PublicKey publicKey) throws at.ac.tuwien.infosys.rnue.helpers.ShareMeException {
        return cryptoimpl.verify(serializable, values, publicKey);
    }
    
}
